from django.urls import path

from . import views

app_name = "evictionsplit"

urlpatterns = [
    path("", views.index, name="index"),
    path("list", views.list_evictions, name="list"),
    path("apply/<int:id>", views.apply, name="apply"),
    path("eviction/<int:id>", views.eviction, name="eviction"),
    path(
        "eviction/<int:eviction_id>/clocking",
        views.change_clocking,
        name="change_clocking",
    ),
    path("eviction/management", views.management, name="manage"),
    path("eviction/<int:eviction_id>/stop", views.stop_eviction, name="stop_eviction"),
]



# Generated by Django 4.0.10 on 2024-01-03 15:56

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("manager", "Can create and manage evictions"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Eviction",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=120)),
                ("active", models.BooleanField(default=True)),
                (
                    "applicants",
                    models.ManyToManyField(
                        related_name="evictions_applied",
                        related_query_name="eviction_applied",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="evictions_created",
                        related_query_name="eviction_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "participants",
                    models.ManyToManyField(
                        blank=True,
                        related_name="evictions_participating",
                        related_query_name="eviction_participating",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Clocking",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("beginning", models.DateTimeField(default=django.utils.timezone.now)),
                ("end", models.DateTimeField(null=True)),
                (
                    "type",
                    models.CharField(
                        choices=[("DS", "Doorstopping"), ("SP", "Standby for ping")],
                        max_length=2,
                    ),
                ),
                (
                    "eviction",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="evictionsplit.eviction",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django.contrib import admin
from django.utils.html import format_html

from .models import Clocking, Eviction


@admin.register(Eviction)
class EvictionAdmin(admin.ModelAdmin):
    fields = ["name", "creator", "active", "participants", "applicants"]
    readonly_fields = ["creator", "participants", "applicants"]

    def save_model(self, request, obj, form, change):
        if getattr(obj, "creator", None) is None:
            obj.creator = request.user
            super().save_model(request, obj, form, change)
            obj.participants.add(request.user)
        super().save_model(request, obj, form, change)


@admin.action(description="Stop selected clocking")
def stop_clockings(modeladmin, request, queryset):
    for clocking in queryset:
        clocking.stop()


class CurrentlyOnDutyFilter(admin.SimpleListFilter):
    title = "currently on duty"
    parameter_name = "on_duty"

    def lookups(self, request, model_admin):
        return [
            ("True", "Active right now"),
            ("False", "Inactive right now"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "True":
            return queryset.filter(end__isnull=True)
        if self.value() == "False":
            return queryset.filter(end__isnull=False)


@admin.register(Clocking)
class ClockingAdmin(admin.ModelAdmin):
    @admin.display(description="Eviction name")
    def eviction_name(self, obj):
        return obj.eviction.name

    @admin.display(description="")
    def _character_pic(self, obj):
        return format_html(
            '<img src="{}" class="img-circle">',
            obj.eviction.creator.profile.main_character.portrait_url(size=32),
        )

    @admin.display(description="Character name")
    def _character_name(self, obj):
        return obj.eviction.creator.profile.main_character.character_name

    fields = ["user", "eviction", "type", ("beginning", "end")]

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ["user", "eviction"]
        else:
            return []

    list_display = [
        "_character_pic",
        "_character_name",
        "type",
        "beginning",
        "end",
        "eviction",
    ]
    list_filter = [
        "eviction",
        "user",
        "type",
        CurrentlyOnDutyFilter,
    ]  # TODO turn the "by user" to a "by character"

    actions = [stop_clockings]



from typing import Optional

from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import get_object_or_404, redirect, render
from django.views.decorators.http import require_POST

from .models import Clocking, Eviction


def add_common_context(request, context: Optional[dict] = None) -> dict:
    """adds the common context used by all view"""
    context = context or {}
    new_context = {
        **{
            "user_has_manager_permission": request.user.has_perm(
                "evictionsplit:manager"
            ),
        },
        **context,
    }
    return new_context


@login_required
@permission_required("evictionsplit.basic_access")
def index(request):
    inactive_error = True if (request.GET.get("error") == "inactive") else False
    user = request.user
    user_active_evictions = user.evictions_participating.filter(active=True)
    for eviction in user_active_evictions:
        eviction.is_standby = eviction.is_user_on_standby(user)
        eviction.is_doorstop = eviction.is_user_on_doorstop(user)
        eviction.time_standby = eviction.user_standby_time(user)
        eviction.time_doorstop = eviction.user_doorstop_time(user)

    return render(
        request,
        "evictionsplit/index.html",
        add_common_context(
            request,
            {
                "inactive_error": inactive_error,
                "user_active_evictions": user_active_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.basic_access")
def list_evictions(request):
    """Lists all active evictions that can be applied to join"""

    active_evictions = Eviction.objects.filter(active=True)
    user_active_evictions = request.user.evictions_participating.filter(active=True)

    return render(
        request,
        "evictionsplit/list.html",
        add_common_context(
            request,
            {
                "active_evictions": active_evictions,
                "user_active_evictions": user_active_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.basic_access")
def apply(request, id):
    eviction = get_object_or_404(Eviction, id=id)
    eviction.user_apply(request.user)

    return redirect("evictionsplit:list")


@login_required
@permission_required("evictionsplit.basic_access")
def eviction(request, id):
    eviction = get_object_or_404(Eviction, id=id)
    if not (eviction.active or request.user.has_perm("evictionsplit:management")):
        return redirect("evictionsplit:index")
    is_standby = eviction.is_user_on_standby(request.user)
    is_doorstop = eviction.is_user_on_doorstop(request.user)
    unknown_change = request.GET.get("unknown")

    participants = []
    for participant in eviction.participants.all():
        participants.append(
            {
                "name": participant.profile.main_character.character_name,
                "portrait_url": participant.profile.main_character.portrait_url,
                "is_standby": eviction.is_user_on_standby(participant),
                "is_doorstop": eviction.is_user_on_doorstop(participant),
            }
        )

    return render(
        request,
        "evictionsplit/eviction.html",
        add_common_context(
            request,
            {
                "eviction": eviction,
                "is_standby": is_standby,
                "is_doorstop": is_doorstop,
                "time_standby": eviction.user_standby_time(request.user),
                "time_doorstop": eviction.user_doorstop_time(request.user),
                "participants": participants,
                "unknown_change": unknown_change,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.basic_access")
@require_POST
def change_clocking(request, eviction_id):
    eviction = Eviction.objects.get(id=eviction_id)
    user = request.user
    if change := request.POST.get("change"):
        if change == "start":
            start = True
        elif change == "stop":
            start = False
        else:
            return redirect("evictionsplit:index")
    if request.POST.get("clocking_type") == Clocking.DOORSTOP:
        if start:
            eviction.user_start_doorstop(user)
        else:
            eviction.user_stop_doorstop(user)
    elif request.POST.get("clocking_type") == Clocking.STANDBY_FOR_PINGS:
        if start:
            eviction.user_start_standby_for_ping(user)
        else:
            eviction.user_stop_standby_for_ping(user)
    else:
        return redirect("evictionsplit:index")

    if request.POST.get("origin") == "eviction":
        return redirect("evictionsplit:eviction", eviction_id)
    else:
        return redirect("evictionsplit:index")


@login_required
@permission_required("evictionsplit.manager")
def management(request):
    active_evictions = Eviction.objects.filter(active=True)
    inactive_evictions = Eviction.objects.filter(active=False)

    return render(
        request,
        "evictionsplit/management.html",
        add_common_context(
            request,
            {
                "active_evictions": active_evictions,
                "inactive_evictions": inactive_evictions,
            },
        ),
    )


@login_required
@permission_required("evictionsplit.manager")
def stop_eviction(request, eviction_id):
    eviction = Eviction.objects.get(id=eviction_id)
    eviction.stop()

    return redirect("evictionsplit:management")



from django.contrib.auth.models import User
from django.test import TestCase

from ..models import Clocking, Eviction


class ClockingTest(TestCase):
    def test_user_is_on_clocked(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        self.assertTrue(not eviction.is_user_on_standby(user1))
        self.assertTrue(not eviction.is_user_on_doorstop(user1))
        eviction.user_start_standby_for_ping(user1)
        eviction.user_start_doorstop(user1)
        self.assertTrue(eviction.is_user_on_standby(user1))
        self.assertTrue(eviction.is_user_on_doorstop(user1))

    def test_user_stop_clocked(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        eviction.user_start_standby_for_ping(user1)
        eviction.user_start_doorstop(user1)
        self.assertTrue(eviction.is_user_on_standby(user1))
        self.assertTrue(eviction.is_user_on_doorstop(user1))
        eviction.user_stop_standby_for_ping(user1)
        eviction.user_stop_doorstop(user1)

        self.assertTrue(not eviction.is_user_on_standby(user1))
        self.assertTrue(not eviction.is_user_on_doorstop(user1))

    def test_no_duplicate_clocking(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        self.assertEquals(0, Clocking.objects.count())

        eviction.user_start_doorstop(user1)
        eviction.user_start_standby_for_ping(user1)

        self.assertEquals(2, Clocking.objects.count())

        eviction.user_start_doorstop(user1)
        eviction.user_start_standby_for_ping(user1)

        self.assertEquals(2, Clocking.objects.count())

    def test_stop_clocking(self):
        user1 = User.objects.create_user("user1", "", "")
        eviction = Eviction.create(
            name="Test eviction",
            creator=user1,
        )

        clocking = Clocking(user=user1, eviction=eviction)
        clocking.save()

        self.assertEquals(None, clocking.end)

        clocking.stop()

        clocking = Clocking.objects.get(eviction=eviction)

        self.assertIsNot(None, clocking.end)
